#include "moglclass.h"
#include "shaders.h"
#include "geometryPositionNormal.h"
#include "simpleCar.h"
#include "simpleSolar.h"
#include "simpleRobot.h"
#include "simpleMobile.h"
#include "light.h"
#include "camera.h"

using namespace moglclass;

#define NUMSHADERS 4

Shader shShaders[NUMSHADERS];
Program spMain;

std::vector<GeometryPositionNormal*> geoList;
//SimpleCar* car;
//SimpleSolar* solar;
//SimpleRobot* robot;
//SimpleMobile* mobile;

bool g_animmode = true;
bool g_wiremode = false;
int g_geometrymode = 0;

float g_fovy = 45.0f;
float g_aspect = 1.0f;
float g_zNear = 1.0f;
float g_zFar = 100.f;

Camera camera1(FLY);
float cameraHeading = 0.0f;
float cameraZ = 10.0f;
glm::vec3 g_eye(0, 0, 10);
glm::vec3 g_at(0, 0, 0);
glm::vec3 g_up(0, 1, 0);

glm::mat4 Identity(1.0f);
glm::mat4 World(1.0f);
glm::mat4 View(1.0f);
glm::mat4 Projection(1.0f);
glm::mat4 MVP(1.0f);

glm::mat4 yRotation(1.0f);

glm::vec4 lightPos = glm::vec4(4, 4, 4, 1);

void init( void )
{
    // load shaders and use the resulting shader program
	spMain.createProgram();
	shShaders[0].loadShader("PerVertexLighting.vs", GL_VERTEX_SHADER);
	shShaders[1].loadShader("PerVertexLighting.fs", GL_FRAGMENT_SHADER);
	spMain.attachShaderToProgram(&shShaders[0]);
	spMain.attachShaderToProgram(&shShaders[1]);
	spMain.linkProgram();
	spMain.useProgram();
	
	Projection = glm::perspective(g_fovy, g_aspect, g_zNear, g_zFar);
	View = camera1.lookAt(g_eye, g_at, g_up);

	// set geometry
	geoList.push_back(new Grid(glm::vec3(0.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(0.0f, 1.0f, 0.0f), 8.0f, 6.0f, 4, 3));
	geoList.push_back(new Cube(glm::vec3(0, 0, 0), 5.0f));
	geoList.push_back(new Sphere(glm::vec3(0, 0, 0), 3.5f, 16, 16));
	geoList.push_back(new Cylinder(glm::vec3(0, 0, 0), 3.5f, 6.0f, 16));
	geoList.push_back(new Torus(glm::vec3(0, 0, 0), 4.0f, 1.0f, 32, 16));
	geoList.push_back(new Parallelepiped(glm::vec3(0.0f, 0.0f, 0.0f), glm::vec3(3.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 3.0f), glm::vec3(4.5f, 7.0f, 0.0f)));

	//car = new SimpleCar();
	//robot = new SimpleRobot();
	//solar = new SimpleSolar();
	//mobile = new SimpleMobile();

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glClearColor( 0.0, 0.0, 0.0, 1.0 ); // black background
}

void cleanup()
{		 
	for (std::vector<GeometryPositionNormal*>::iterator i = geoList.begin(); i != geoList.end(); ++i)
	{
		delete (*i);
		i = geoList.erase(i);
	}

	//if (car) delete car;
	//if (robot) delete robot;
	//if (solar) delete solar;
	//if (mobile) delete mobile;
}

void display( void )
{
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );     // clear the window

	// set drawmode
	if (g_wiremode) 
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);  // draw wireframe 
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	// draw filled 

	// MVP matrix
	View = camera1.View();

	// draw 1st geometry
	World = yRotation;
	//float angle = DEGTORAD(cameraHeading);
	//World = glm::rotate(glm::mat4(1.0f), -angle, glm::vec3(0, 1, 0)) * glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, -cameraZ));

	MVP = Projection * View * World;
	spMain.useProgram();
	spMain.setUniform("gMVP", MVP);
	spMain.setUniform("gProjection", Projection);
	spMain.setUniform("gView", View);
	spMain.setUniform("gModel", World);

	glm::vec3 cameraPos = camera1.getPosition();
	spMain.setUniform("gEyePosition", cameraPos);
	//glm::vec4 lightPos = glm::vec3(4, 4, 4, 1);
	spMain.setUniform("gLightPosition", lightPos);
	// material color
	glm::vec3 geoColor = geoList[g_geometrymode]->Color();
	spMain.setUniform("gMaterialAmbientColor", glm::vec4(0.2, 0.2, 0.2, 1.0));
	spMain.setUniform("gMaterialDiffuseColor", glm::vec4(geoColor.r, geoColor.g, geoColor.b, 1.0));
	spMain.setUniform("gMaterialSpecularColor", glm::vec4(1.0, 1.0, 1.0, 1.0));
	spMain.setUniform("gMaterialSpecularShininess", 5.0f);

	geoList[g_geometrymode]->draw();

	// draw geometry
	//World = Identity;  //yRotation2;
	//MVP = Projection * View * World;
	//car->draw(&spMain, Projection, View, Identity);
	//solar->draw(&spMain, Projection, View, Identity);
	//robot->draw(&spMain, Projection, View, Identity);
	//mobile->draw(&spMain, Projection, View, World);

	glBindVertexArray(0);
	glDisableVertexAttribArray(0);

    glutSwapBuffers();
}

void update()
{
	static int currentTime, deltaTime, prevTime = 0; 
	static int startTime, elapsedTime = 0;
	currentTime = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = currentTime - prevTime;
	prevTime = currentTime;

	if (g_animmode)
	{
		// translate & rotate around x/y axis
		static float angle = 0.0f;
		angle += 0.001f;

		// geometry: translate & rotate in x & rotate in y
		glm::mat4 T = glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, 1));
		glm::mat4 R = glm::rotate(T, angle, glm::vec3(1, 0, 0));
		yRotation = glm::rotate(R, angle, glm::vec3(0, 1, 0));

		// each geometry update
		//geoList[0]->update((float)deltaTime);

		// car geometry update
		//car->update((float)deltaTime);
		//robot->update((float)deltaTime);
		//solar->update((float)deltaTime);
		//mobile->update((float)deltaTime);
	}

	elapsedTime = currentTime - startTime;
	glutPostRedisplay();
}
void keyboard( unsigned char key, int x, int y )
{
	switch (key)  
	{    
	case 0x1B:    
	case 'q':    
	case 'Q':      
		exit(0);     
	case 'a':
		g_animmode = !g_animmode;
		break;
	case ' ':
		g_wiremode = !g_wiremode;
		break;  
	case 'g':
		g_geometrymode = (g_geometrymode+1) % 6;
		printf("g_geometrymode=%d\n", g_geometrymode);
		break;
	}
	glutPostRedisplay();
}
/*
void specialkey(int key, int x, int y )
{
	switch (key)  
	{    
	case GLUT_KEY_LEFT:
		cameraHeading -= 10;
		//printf("cameraHeading=%f\n", cameraHeading);

		//g_eye.x += 0.1f;
		//g_at.x += 0.1f;
		//mTranslate[0][1] -= 0.1f;
		//robot->setTheta(g_theta-=10.0f);
		break;  
	case GLUT_KEY_RIGHT:    
		cameraHeading += 10;
		//printf("cameraHeading=%f\n", cameraHeading);

		//g_eye.x += 0.1f;
		//g_at.x += 0.1f;
		//mTranslate[0][1] += 0.1f;
		//robot->setTheta(g_theta+=10.0f);
		break;  
	case GLUT_KEY_UP:      
		cameraZ += 1.0f;
		printf("cameraZ=%f\n", cameraZ);

		//g_eye.z -= 0.1f;
		//g_at.z -= 0.1f;
		//mTranslate[0][1] -= 0.1f;
		//robot->setPhi(g_phi+=10.0f);
		break;  
	case GLUT_KEY_DOWN:
		cameraZ -= 1.0f;
		printf("cameraZ=%f\n", cameraZ);

		//g_eye.z += 0.1f;
		//g_at.z += 0.1f;
		//mTranslate[0][1] -= 0.1f;
		//robot->setPhi(g_phi-=10.0f);
		break;  
	case GLUT_KEY_PAGE_UP:      
		//g_eye.z -= 0.1f;
		//g_at.z -= 0.1f;
		//mTranslate[0][1] -= 0.1f;
		//robot->setPsi(g_psi+=10.0f);
		break;  
	case GLUT_KEY_PAGE_DOWN:
		//g_eye.z += 0.1f;
		//g_at.z += 0.1f;
		//mTranslate[0][1] -= 0.1f;
		//robot->setPsi(g_psi-=10.0f);
		break;  
	}
	glutPostRedisplay();
}
*/

void specialkey(int key, int x, int y)
{
 if (key == GLUT_KEY_F1)   // x-movement
        camera1.strafe(0.5);
 else if (key == GLUT_KEY_F2)
        camera1.strafe(-0.5);
 else if (key == GLUT_KEY_F3) // y-movement
        camera1.fly(0.5);
 else if (key == GLUT_KEY_F4)
        camera1.fly(-0.5);
 else if (key == GLUT_KEY_F5) // z-movement
        camera1.walk(0.5);
    else if (key == GLUT_KEY_F6)
        camera1.walk(-0.5);

 else if (key == GLUT_KEY_F7) // yaw (by y-axis)
        camera1.yaw(2.5);
 else if (key == GLUT_KEY_F8)
        camera1.yaw(-2.5);
 else if (key == GLUT_KEY_F9) // pitch (by x-axis)
        camera1.pitch(2.5);
    else if (key == GLUT_KEY_F10)
        camera1.pitch(-2.5);
 else if (key == GLUT_KEY_HOME) // roll (by z-axis)
        camera1.roll(2.5);
    else if (key == GLUT_KEY_END)
        camera1.roll(-2.5);

 else if (key == GLUT_KEY_LEFT) // same as town
        camera1.yaw(2.5);
 else if (key == GLUT_KEY_RIGHT)
        camera1.yaw(-2.5);
 else if (key == GLUT_KEY_UP) 
        camera1.walk(0.5);
    else if (key == GLUT_KEY_DOWN)
        camera1.walk(-0.5);

 glutPostRedisplay();
}


void mouse(int button, int state, int mx, int my)
{
	if (button==GLUT_RIGHT_BUTTON && state==GLUT_DOWN)
		exit(0);
}

void motion(int mx, int my)
{
	int w = glutGet(GLUT_WINDOW_WIDTH);
	int h = glutGet(GLUT_WINDOW_HEIGHT);
	float x = (float) (mx - w*0.5) / w;
	float y = (float) (h*0.5 - my) / h;
	//fprintf(stderr, "x=%f y=%f\n", x, y);
	//mTranslate = glm::translate(glm::mat4(1.0f), glm::vec3(x, y, 0)); 

	glutPostRedisplay();
}

void reshape(int w, int h)
{
	g_aspect = (float) (w/h);

    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glutPostRedisplay();
}

int main( int argc, char **argv )
{
	glutInit(&argc, argv);
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
    glutInitWindowSize( SCREEN_WIDTH, SCREEN_HEIGHT );
    glutCreateWindow(argv[0]);
    
	glewInit();
    init();

    glutDisplayFunc( display );
    glutKeyboardFunc( keyboard );
    glutSpecialFunc( specialkey );
	glutMouseFunc( mouse ); 
	glutMotionFunc( motion ); 
	glutIdleFunc( update );
	glutReshapeFunc( reshape );

    glutMainLoop();

	cleanup();

    return 0;
}
