//////////////////////////////////////////////////////////////////////
// car.cpp: implementation for the Car class (Hierarchical Transformation).
// -Kyoung Shin Park (2013 Spring)
//////////////////////////////////////////////////////////////////////
#include "simpleCar.h"
#include "color.h"

namespace moglclass
{
	
SimpleCar::SimpleCar()
{
	angle = 0.0f; // 0~360
	direction = 1.0f; // +/-1
	position = glm::vec3(-5.0f, 0.0f, 0.0f);

	init();
}

SimpleCar::~SimpleCar()
{ 
	if (body) delete body;
	if (wheel) delete wheel;
	std::cout << "~SimpleCar" << std::endl; 
}

void SimpleCar::init()
{
	body = new Parallelepiped(glm::vec3(0.0f, 0.0f, 0.0f), glm::vec3(-4.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 4.0f), glm::vec3(0.0f, 2.0f, 0.0f));
	wheel = new Torus(glm::vec3(0.0f, 0.0f, .0f), 0.6f, 0.3f, 32, 16);
}

void SimpleCar::draw(Program* p, glm::mat4& projection, glm::mat4& view, glm::mat4& model)
{
	if (p == NULL) return;

	p->useProgram();
	p->setUniform("gProjection", projection);
	p->setUniform("gView", view);

	glm::mat4 carMatrix = model * bodyTransform;
	p->setUniform("gModel", carMatrix);
	p->setUniform("gMaterialDiffuseColor", Color::Red);
	if (body) body->draw();

	p->setUniform("gMaterialDiffuseColor", Color::Green);
	glm::mat4 wheelMatrix = model * bodyTransform * wheelTransform[0];
	p->setUniform("gModel", wheelMatrix);
	if (wheel) wheel->draw();
	wheelMatrix = model * bodyTransform * wheelTransform[1];
	p->setUniform("gModel", wheelMatrix);
	if (wheel) wheel->draw();

	p->setUniform("gMaterialDiffuseColor", Color::Blue);
	wheelMatrix = model * bodyTransform * wheelTransform[2];
	p->setUniform("gModel", wheelMatrix);
	if (wheel) wheel->draw();
	wheelMatrix = model * bodyTransform * wheelTransform[3];
	p->setUniform("gModel", wheelMatrix);
	if (wheel) wheel->draw();
}

bool SimpleCar::update(float deltaTime)
{
	if (active)
	{
		angle = angle - 180.0f * (float) (deltaTime) * 0.001f * direction;
		position[0] = position[0] + (float) (deltaTime) * 0.001f * direction;

		//std::cout << "position[0]=" << position[0] << std::endl;
		if (position[0] * direction > 5)
			direction = -direction;
	}

	bodyTransform = glm::translate(glm::mat4(1.0f), position); // RHS x+ right
	wheelTransform[0] = glm::translate(glm::mat4(1.0f), glm::vec3(-3.5f, 0.0f, 0.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f)); 
	wheelTransform[1] = glm::translate(glm::mat4(1.0f), glm::vec3(-3.5f, 0.0f, 3.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f)); 
	wheelTransform[2] = glm::translate(glm::mat4(1.0f), glm::vec3(-0.5f, 0.0f, 0.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f)); 
	wheelTransform[3] = glm::translate(glm::mat4(1.0f), glm::vec3(-0.5f, 0.0f, 3.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f)); 

	return true;
}

} // moglclass namespace